import datetime
from typing import Dict, Optional, List, Union

from moviebotapi import Session
from moviebotapi.core import utils
from moviebotapi.core.models import MediaType
from moviebotapi.core.utils import json_object


@json_object
class Person:
    id: int
    douban_id: int
    imdb_id: str
    cn_name: str
    en_name: str
    aka_cn_names: str
    aka_en_names: str
    country: str
    date_of_birth: str
    douban_image_url: str
    description: str
    role: str
    duty: str
    order_number: int

    def __init__(self, data: Dict):
        utils.copy_value(data, self, True)


@json_object
class Season:
    name: str
    season_number: int
    episode_count: int
    intro: str
    air_date: int
    poster_url: str
    douban_id: int

    def __init__(self, data: Dict):
        utils.copy_value(data, self, True)


@json_object
class MediaMeta:
    id: int
    douban_id: int
    tmdb_id: int
    imdb_id: str
    tvdb_id: int
    # 数据最后更新时间
    gmt_modified: datetime.datetime
    media_type: MediaType
    title: str
    en_title: str
    original_title: str
    intro: str
    rating: float
    release_year: int
    # 首映日期
    premiere_date: int
    # 影片时常，单位分钟
    duration: int
    # 封面图
    poster_url: str
    # 背景图
    background_url: str
    genres: List[str]
    country: List[str]
    # 剧集时包含
    season_list: List[Season]

    def __init__(self, data: Dict):
        utils.copy_value(data, self, True)


class MetaApi:
    """
    自建数据操作接口
    请勿恶意调用，识别到违规刷数据行为直接封禁License
    """

    def __init__(self, session: Session):
        self._session: Session = session

    def get_cast_crew_by_douban(self, media_type: MediaType, douban_id: int):
        """
        根据豆瓣编号获取影片全部演员信息
        """
        list_ = self._session.get('meta.get_cast_crew_by_douban', {
            'media_type': media_type.value,
            'douban_id': douban_id
        })
        if not list_:
            return
        return [Person(x) for x in list_]

    def get_cast_crew_by_tmdb(self, media_type: MediaType, tmdb_id: int, season_number: Optional[int] = None) -> \
            Optional[List[Person]]:
        """
        根据tmdb信息获取影片全部演员信息
        """
        list_ = self._session.get('meta.get_cast_crew_by_tmdb', {
            'media_type': media_type.value,
            'tmdb_id': tmdb_id,
            'season_number': season_number
        })
        if not list_:
            return
        return [Person(x) for x in list_]

    def get_media_by_tmdb(self, media_type: MediaType, tmdb_id: int) -> Optional[Union[MediaMeta]]:
        """
        根据tmdb编号获取自建影视元数据
        """
        res = self._session.get('meta.get_media_by_tmdb_id', {
            'media_type': media_type.value,
            'tmdb_id': tmdb_id
        })
        if not res:
            return
        return MediaMeta(res)

    def get_media_by_douban(self, media_type: MediaType, douban_id: int):
        """
        根据豆瓣编号获取自建影视元数据
        """
        res = self._session.get('meta.get_media_by_douban_id', {
            'media_type': media_type.value,
            'douban_id': douban_id
        })
        if not res:
            return
        return MediaMeta(res)

    def share_meta_from_id(self, tmdb_id: int, douban_id: int, season_number: Optional[int] = None):
        """
        根据TMDB和豆瓣的影片编号，共享数据到自建服务器
        :param tmdb_id: TMDB编号
        :param douban_id: 豆瓣编号
        :param season_number: 如果为剧集时，可以选择强制指定季，如果不填，则自动取豆瓣
        :return:
        """
        return self._session.get('meta.share_meta_from_id', {
            'tmdb_id': tmdb_id,
            'douban_id': douban_id,
            'season_number': season_number
        })



from moviebotapi.core import utils


class BaseModel(object):

    def to_json(self, hidden_fields=None):
        """
        Json序列化
        :param hidden_fields: 覆盖类属性 hidden_fields
        :return:
        """
        model_json = {}
        if not hidden_fields:
            hidden_fields = []
        for column in self.__dict__:
            if column in hidden_fields:
                continue
            if hasattr(self, column):
                model_json[column] = utils.parse_field_value(getattr(self, column))
        if '_sa_instance_state' in model_json:
            del model_json['_sa_instance_state']
        return model_json



import datetime
import decimal
import json
import os
import re
from enum import Enum
from typing import Dict, List, _GenericAlias, Optional

import cn2an

from moviebotapi.core.models import MediaType


def trans_unit_to_mb(size: float, unit: str) -> float:
    """
    按文件大小尺寸规格，转换成MB单位的数字
    :param size:
    :param unit:
    :return:
    """
    if unit == 'GB' or unit == 'GiB':
        return round(size * 1024, 2)
    elif unit == 'MB' or unit == 'MiB':
        return round(size, 2)
    elif unit == 'KB' or unit == 'KiB':
        return round(size / 1024, 2)
    elif unit == 'TB' or unit == 'TiB':
        return round(size * 1024 * 1024, 2)
    elif unit == 'PB' or unit == 'PiB':
        return round(size * 1024 * 1024 * 1024, 2)
    else:
        return size


def trans_size_str_to_mb(size: str):
    """
    把一个字符串格式的文件尺寸单位，转换成MB单位的标准数字
    :param size:
    :return:
    """
    if not size:
        return 0.0
    s = None
    u = None
    if size.find(' ') != -1:
        arr = size.split(' ')
        s = arr[0]
        u = arr[1]
    else:
        if size.endswith('GB'):
            s = size[0:-2]
            u = 'GB'
        elif size.endswith('GiB'):
            s = size[0:-3]
            u = 'GB'
        elif size.endswith('MB'):
            s = size[0:-2]
            u = 'MB'
        elif size.endswith('MiB'):
            s = size[0:-3]
            u = 'MB'
        elif size.endswith('KB'):
            s = size[0:-2]
            u = 'KB'
        elif size.endswith('KiB'):
            s = size[0:-3]
            u = 'KB'
        elif size.endswith('TB'):
            s = size[0:-2]
            u = 'TB'
        elif size.endswith('TiB'):
            s = size[0:-3]
            u = 'TB'
        elif size.endswith('PB'):
            s = size[0:-2]
            u = 'PB'
        elif size.endswith('PiB'):
            s = size[0:-3]
            u = 'PB'
    if not s:
        return 0.0
    if s.find(',') != -1:
        s = s.replace(',', '')
    return trans_unit_to_mb(float(s), u)


def parse_field_value(field_value):
    if isinstance(field_value, decimal.Decimal):  # Decimal -> float
        field_value = round(float(field_value), 2)
    elif isinstance(field_value, datetime.datetime):  # datetime -> str
        field_value = str(field_value)
    elif isinstance(field_value, list):
        field_value = [parse_field_value(i) for i in field_value]
    if hasattr(field_value, 'to_json'):
        field_value = field_value.to_json()
    elif isinstance(field_value, Enum):
        field_value = field_value.name
    elif isinstance(field_value, Dict):
        val = {}
        for key_ in field_value:
            val[key_] = parse_field_value(field_value[key_])
        field_value = val
    return field_value


def json_object(cls):
    def to_json(self):
        """
        Json序列化
        :param hidden_fields: 覆盖类属性 hidden_fields
        :return:
        """

        model_json = {}

        for column in self.__dict__:
            if hasattr(self, column):
                model_json[column] = parse_field_value(getattr(self, column))
        if '_sa_instance_state' in model_json:
            del model_json['_sa_instance_state']
        return model_json

    cls.to_json = to_json

    return cls


def string_to_number(text):
    """
    文本转化成字符串，支持中文大写数字，如一百二十三
    :param text:
    :return:
    """
    if text is None:
        return None
    if text.isdigit():
        return int(text)
    else:
        try:
            return cn2an.cn2an(text)
        except ValueError as e:
            return None


def name_convert_to_camel(name: str) -> str:
    """下划线转驼峰(小驼峰)"""
    return re.sub(r'(_[a-z])', lambda x: x.group(1)[1].upper(), name)


def copy_value(source: Dict, target: object, camel_case=False) -> None:
    if not source:
        return
    if not target or not target.__annotations__:
        return
    for name in target.__annotations__:
        anno = target.__annotations__[name]
        setattr(target, name, parse_value(anno, source.get(name_convert_to_camel(name) if camel_case else name)))


def to_dict(obj: object) -> Optional[Dict]:
    if not obj or not obj.__annotations__:
        return
    result = {}
    for name in obj.__annotations__:
        anno = obj.__annotations__[name]
        result.update({name: parse_value(anno, getattr(obj, name))})
    return result


def _list_value(value):
    if isinstance(value, str):
        if value[0] in ['{', '[']:
            return json.loads(value)
        else:
            return value.split(',')
    else:
        return list(value)


def _dict_value(value):
    if isinstance(value, str):
        return json.loads(value)
    else:
        return value


def parse_value(func, value, default_value=None):
    if value is not None:
        if func == bool:
            if value in (1, True, "1", "true"):
                return True
            elif value in (0, False, "0", "false"):
                return False
            else:
                raise ValueError(value)

        elif func in (int, float):
            try:
                if isinstance(value, str):
                    value = value.replace(',', '')
                return func(value)
            except ValueError:
                return float('nan')
        elif func == datetime.datetime:
            if isinstance(value, datetime.datetime):
                return value
            elif isinstance(value, str):
                if value:
                    return datetime.datetime.strptime(value, '%Y-%m-%d %H:%M:%S')
                else:
                    return None
            else:
                return None
        elif func in [Dict, dict]:
            return _dict_value(value)
        elif func in [List, list]:
            return _list_value(value)
        elif func == MediaType:
            return MediaType.get(value)
        elif isinstance(func, _GenericAlias):
            if func.__origin__ in [List, list]:
                list_ = _list_value(value)
                res = []
                for x in list_:
                    res.append(parse_value(func.__args__[0], x))
                return res
        return func(value)
    else:
        return default_value


"""文件后缀"""
EXT_TYPE = {
    'info': ['.nfo', '.txt', '.cue'],
    'subtitle': ['.ass', '.srt', '.smi', '.ssa', '.sub', '.vtt', '.idx'],
    'image': ['.jpg', '.jpeg', '.png', '.gif', '.bmp', '.ico', '.tif'],
    'video': ['.mp4', '.mkv', '.avi', '.wmv', '.mpg', '.mpeg', '.mov', '.rm', '.rmvb', '.ram', '.flv', '.ts',
              '.iso', '.m2ts', '.bdmv'],
    'audio': ['.mka', '.mp3', '.m3u', '.flac']
}


def get_file_type(filepath):
    """
    获取一个文件的格式类型，基于后缀判断
    :param filepath:
    :return: info、subtitle、image、video、audio
    """
    if not EXT_TYPE:
        return 'unknown'
    ext = os.path.splitext(filepath)[-1].lower()
    for key in EXT_TYPE.keys():
        if ext in EXT_TYPE[key]:
            return key
    return 'unknown'


class _Countries:
    """ISO3166国家码处理工具"""
    iso3166_mapping: dict = dict()

    def __init__(self):
        with open(os.path.join(os.path.split(os.path.realpath(__file__))[0], 'country_mapping.txt'), 'r') as f:
            for line in f:
                line = line.strip('\n')
                if line == '':
                    continue
                arr = line.split('=')
                self.iso3166_mapping[arr[0]] = arr[1]

    def get(self, code):
        code = str(code).upper()
        if code in self.iso3166_mapping.keys():
            return self.iso3166_mapping[code]
        else:
            if '其他' in self.iso3166_mapping:
                return self.iso3166_mapping['其他']
            else:
                return '其他'


Countries = _Countries()
CN_EP_PATTERN = '[集话回話画期]'
TV_NUM_PATTERN = '[1234567890一二三四五六七八九十]{1,4}'
SEASON_NUM_PATTERN = '[1234567890一二三四五六七八九十]{1,3}'

SEASON_PATTERNS = [
    re.compile('[sS](%s)?[-—~]{1,3}[sS]?(%s)' % (SEASON_NUM_PATTERN, SEASON_NUM_PATTERN)),
    re.compile('第(%s)[季部辑]?分?[-—~]{1,3}第?(%s)[季部辑]分?' % (SEASON_NUM_PATTERN, SEASON_NUM_PATTERN)),
    re.compile(r'S(?:eason)?(\d{1,3})', re.IGNORECASE),
    re.compile(r'S(?:eason)?[-]{0,3}(\d{1,3})', re.IGNORECASE),
    re.compile('第(%s)[季部辑]分?' % SEASON_NUM_PATTERN),
    re.compile(r'S(?:eason)?\s(\d{1,3})', re.IGNORECASE)
]
COMPLETE_SEASON_PATTERNS = [
    re.compile('全(%s)[季部辑]' % SEASON_NUM_PATTERN)
]
EPISODE_PATTERNS = [
    re.compile(r'[Ee][Pp]?(%s)' % TV_NUM_PATTERN),
    re.compile('第?(%s)%s' % (TV_NUM_PATTERN, CN_EP_PATTERN)),
    re.compile(r'第\s?(%s)\s?%s' % (TV_NUM_PATTERN, CN_EP_PATTERN)),
    re.compile(r'^(%s)\s?$' % TV_NUM_PATTERN),
    re.compile(r'^(%s)\.' % TV_NUM_PATTERN),
    re.compile(r'(%s)[oO][fF]%s' % (TV_NUM_PATTERN, TV_NUM_PATTERN)),
    re.compile(r'[\[【](%s)[】\]]' % TV_NUM_PATTERN),
    re.compile(r'\s{1}-\s{1}(%s)' % TV_NUM_PATTERN)
]
EPISODE_RANGE_PATTERNS = [
    re.compile('第(%s)%s?-第?(%s)%s' % (TV_NUM_PATTERN, CN_EP_PATTERN, TV_NUM_PATTERN, CN_EP_PATTERN)),
    re.compile(r'[Ee][Pp]?(\d{1,4})[Ee][Pp]?(\d{1,4})'),
    re.compile(r'[Ee][Pp]?(\d{1,4})-[Ee]?[Pp]?(\d{1,4})'),
    re.compile(r'^(%s)[-到](%s)$' % (TV_NUM_PATTERN, TV_NUM_PATTERN)),
    re.compile(r'^(%s)\s{0,4}-\s{0,4}.+$' % TV_NUM_PATTERN),
    re.compile(r'[\[【\(](%s)-(%s)[】\]\)]' % (TV_NUM_PATTERN, TV_NUM_PATTERN)),
    re.compile(r'(全)(%s)%s' % (TV_NUM_PATTERN, CN_EP_PATTERN))
]
COMPLETE_EPISODE_PATTERNS = [
    re.compile('全(%s)%s' % (TV_NUM_PATTERN, CN_EP_PATTERN)),
    re.compile('(%s)%s全' % (TV_NUM_PATTERN, CN_EP_PATTERN)),
    re.compile('全%s' % CN_EP_PATTERN),
    re.compile('所有%s' % CN_EP_PATTERN)
]


class MediaParser:
    """媒体信息格式化工具类，待重构完成"""

    @staticmethod
    def parse_episode(text, match_single=True, match_range=True):
        if not text:
            return
        if get_file_type(text) != 'unknown':
            text = os.path.splitext(text)[0]
        ep_index_start = None
        ep_index_end = None
        ep_str = None
        pts = []
        if match_range:
            pts += EPISODE_RANGE_PATTERNS
        if match_single:
            pts += EPISODE_PATTERNS
        for p in pts:
            m = p.search(text)
            if m:
                ep_str = m.group()
                if len(m.groups()) == 1:
                    ep_index_start = string_to_number(m.group(1))
                    ep_index_end = None
                elif len(m.groups()) == 2:
                    ep_index_start = string_to_number(m.group(1))
                    ep_index_end = string_to_number(m.group(2))
                if ep_index_start and ep_index_start > 2000:
                    # is year,not episode
                    ep_index_start = None
                    ep_index_end = None
                    continue
                break
        if ep_index_start is None:
            return
        return {'start': ep_index_start, 'end': ep_index_end, 'text': ep_str}

    @staticmethod
    def parse_season(text):
        season_start = None
        season_end = None
        season_text = None
        for p in SEASON_PATTERNS:
            m = p.search(text)
            if m:
                season_text = m.group()
                if season_text == text:
                    season_text = m.group(1)
                if len(m.groups()) == 1:
                    season_start = string_to_number(m.group(1))
                    season_end = None
                elif len(m.groups()) == 2:
                    season_start = string_to_number(m.group(1))
                    season_end = string_to_number(m.group(2))
                break
        return {'start': season_start, 'end': season_end, 'text': season_text}

    @staticmethod
    def episode_format(episode, prefix=''):
        if not episode:
            return
        if isinstance(episode, str):
            episode = list(filter(None, episode.split(',')))
            episode = [int(i) for i in episode]
        elif isinstance(episode, int):
            episode = [episode]
        if episode:
            episode.sort()
        if len(episode) <= 2:
            return ','.join([str(e).zfill(2) for e in episode])
        else:
            episode.sort()
            return '%s%s-%s%s' % (prefix, str(episode[0]).zfill(2), prefix, str(episode[len(episode) - 1]).zfill(2))

    @staticmethod
    def trim_season(string):
        if not string:
            return
        string = str(string)
        season = MediaParser.parse_season(string)
        if season and season.get('text'):
            simple_name = string.replace(season.get('text'), '').strip()
        else:
            simple_name = string
        return simple_name



import datetime
from typing import Dict, List, Optional

from moviebotapi.core import utils
from moviebotapi.core.session import Session
from moviebotapi.core.utils import json_object


@json_object
class User:
    gmt_create: datetime.datetime
    gmt_modified: datetime.datetime
    username: str
    nickname: str
    douban_user: str
    qywx_user: str
    telegram_user_id: int
    bark_url: str
    avatar: str
    role: int
    pushdeer_key: str
    score_rule_name: str
    password: str

    def __init__(self, data: Dict, api: 'UserApi'):
        utils.copy_value(data, self)
        self._api = api
        self.uid: int = utils.parse_value(int, data.get('id'))

    def delete(self):
        self._api.delete(self.uid)

    def reset_password(self, new_password: str):
        self._api.reset_password(self.uid, new_password)

    def update(self):
        self._api.update(self)


class UserApi:
    def __init__(self, session: Session):
        self._session: Session = session

    def reset_password(self, uid: int, new_password: str):
        self._session.post('user.reset_password', json={'uid': uid, 'password': new_password})

    def delete(self, uid: int):
        self._session.post('user.delete_user', json={'uid': uid})

    def get(self, uid: int) -> Optional[User]:
        user = self._session.get('user.get_user', params={
            'id': uid
        })
        if not user:
            return
        return User(user, self)

    def update(self, user: User):
        self._session.post('user.update_user', json={
            'uid': user.uid,
            'username': user.username,
            'nickname': user.nickname,
            'new_password': user.password,
            'role': user.role,
            'douban_user': user.douban_user,
            'qywx_user': user.qywx_user,
            'pushdeer_key': user.pushdeer_key,
            'bark_url': user.bark_url,
            'score_rule_name': user.score_rule_name,
            'telegram_user_id': user.telegram_user_id
        })

    def list(self) -> List[User]:
        list_ = self._session.get('user.get_user_list')
        if not list_:
            return []
        return [User(x, self) for x in list_]

    def upload_img_to_cloud_by_filepath(self, filepath: str):
        return self._session.get('user.upload_img_to_cloud_by_filepath', {
            'filepath': filepath
        })



from typing import List, Dict, Optional

from moviebotapi import Session
from moviebotapi.core import utils
from moviebotapi.core.basemodel import BaseModel
from moviebotapi.core.models import MediaType


class MediaNameMeta(BaseModel):
    filepath: Optional[str] = None
    cn_name: str = None
    aka_names: List[str] = []
    en_name: str = None
    year: int = None
    season_number: List[int] = None
    episode_number: List[int] = None
    resolution: str = None
    media_source: str = None
    media_codec: str = None
    media_audio: List[str] = None
    release_team: str = None
    tmdb_id: int = None
    media_type: MediaType = None

    def __init__(self, data: Dict = None):
        utils.copy_value(data, self)


class TVMeta(BaseModel):
    season_start: int = None
    season_end: int = None
    season_full_index: List[int] = []
    ep_start: int = None
    ep_end: int = None
    ep_full_index: List[int] = []
    season_is_fill: bool = False
    ep_is_fill: bool = False
    contains_complete_ep: bool = False
    contains_complete_season: bool = False
    contains_multiple_season: bool = False

    def __init__(self, data: Dict = None):
        utils.copy_value(data, self)


class MetaSearchResult(BaseModel):
    tmdb_id: int = None
    douban_id: int = None
    cn_name: str = None
    original_name: str = None
    release_year: int = None
    release_date: str = None
    media_type: MediaType = None
    season_number: int = None

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


class AmrApi:
    """
    Automatic media Recognition
    自动媒体信息分析接口
    """

    def __init__(self, session: Session):
        self._session: Session = session

    def parse_name_meta_by_string(self, string: str) -> Optional[MediaNameMeta]:
        """
        根据一个字符串（种子名、文件名等）解析出名称中包含的影片信息
        """
        res = self._session.get('amr.parse_name_meta_by_string', {
            'string': string
        })
        if not res:
            return
        return MediaNameMeta(res)

    def parse_name_meta_by_filepath(self, filepath: str) -> Optional[MediaNameMeta]:
        """
        根据一个影片文件路径解析出影片信息
        需要为完整路径，会根据分析上级目录的有效信息进行解析
        """
        res = self._session.get('amr.parse_name_meta_by_filepath', {
            'filepath': filepath
        })
        if not res:
            return
        return MediaNameMeta(res)

    def analysis_string(self, string: str):
        """
        根据一个字符串（种子名、文件名等），分析出它对应的影片信息，包括TMDBID
        """
        res = self._session.get('amr.analysis_string', {
            'string': string
        })
        if not res:
            return
        return MetaSearchResult(res)

    def analysis_filepath(self, filepath: str):
        """
        根据一个影片文件路径，分析出它对应的影片信息，包括TMDBID
        """
        res = self._session.get('amr.analysis_filepath', {
            'string': filepath
        })
        if not res:
            return
        return MetaSearchResult(res)

    def analysis_douban_meta(self, cn_name: Optional[str], en_name: Optional[str] = None, year: Optional[int] = None,
                             season_number: Optional[int] = None):
        """
        根据提供的信息，分析出豆瓣的关联影片
        """
        res = self._session.get('amr.analysis_douban_meta', {
            'cn_name': cn_name,
            'en_name': en_name,
            'year': year,
            'season_number': season_number
        })
        if not res:
            return
        return MetaSearchResult(res)



from typing import List, Dict

from moviebotapi import Session
from moviebotapi.core import utils
from moviebotapi.core.basemodel import BaseModel
from moviebotapi.core.models import MediaType
from moviebotapi.core.utils import json_object


@json_object
class SubtitleStream:
    """字幕流信息"""
    codec: str
    language: str
    display_language: str
    display_title: str
    # 外部字幕
    external: bool
    is_default: bool

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


@json_object
class AudioStream:
    """音频流信息"""
    codec: str
    language: str
    display_language: str
    display_title: str
    is_default: bool
    channel_layout: str

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


@json_object
class MediaItem:
    """媒体服务器的影片基础模型"""
    tmdb_id: int
    imdb_id: str
    tvdb_id: str
    url: str
    id: str
    name: str
    # 剧集才有，集号
    index: int
    type: MediaType
    poster_url: str
    thumb_url: str
    backdrop_url: str

    # 视频容器类型 mkv 原盘
    video_container: str
    # 视频编码
    video_codec: str
    # 视频分辨率
    video_resolution: str
    # 字幕流
    subtitle_streams: List[SubtitleStream]
    # 音频流
    audio_streams: List[AudioStream]
    sub_items: list
    # 播放状态
    status: int

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


class MediaFolder(BaseModel):
    """媒体服务器配置的影音库文件夹"""
    id: str
    name: str
    path: str
    sub_folders: list


ListMediaItem = List[MediaItem]
ListMediaFolder = List[MediaFolder]


class MediaServerApi:
    def __init__(self, session: Session):
        self._session: Session = session

    def list_episodes_from_tmdb(self, tmdb_id: int, season_number: int, fetch_all: bool = False):
        items = self._session.get('media_server.list_episodes_from_tmdb', {
            'tmdb_id': tmdb_id,
            'season_number': season_number,
            'fetch_all': fetch_all
        })
        if not items:
            return []
        return [MediaItem(x) for x in items]

    def search_by_tmdb(self, tmdb_id: int, fetch_all: bool = False):
        items = self._session.get('media_server.search_by_tmdb', {
            'tmdb_id': tmdb_id,
            'fetch_all': fetch_all
        })
        if not items:
            return []
        return [MediaItem(x) for x in items]



from enum import Enum
from typing import Dict, List, Optional

from moviebotapi import Session
from moviebotapi.core import utils
from moviebotapi.core.models import MediaType
from moviebotapi.core.utils import json_object
from moviebotapi.subscribe import SubStatus


@json_object
class DoubanPeople:
    """剧组人员、导演、演员等人员信息"""
    douban_id: int
    tmdb_id: int
    imdb_id: str
    name: str
    en_name: str
    role: str
    pic_url: str
    type: str

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


@json_object
class DoubanMedia:
    id: int
    url: str
    release_year: str
    local_name: str
    en_name: str
    cover_image: str
    rating: float
    user_id: str
    intro: str
    director: List[DoubanPeople]
    actor: List[DoubanPeople]
    duration: int
    trailer_video_url: str

    def __init__(self, data: Dict):
        utils.copy_value(data, self)
        # 对历史遗留接口不规范命名做更正
        self.media_type: MediaType = utils.parse_value(MediaType, data.get('type'))
        self.aka_names: List[str] = utils.parse_value(List[str], data.get('alias'))
        self.genres: List[str] = utils.parse_value(List[str], data.get('cates'))
        self.country: List[str] = utils.parse_value(List[str], data.get('area'))
        self.cn_name: str = utils.parse_value(str, data.get('name'))
        self.premiere_date: str = utils.parse_value(str, data.get('release_date'))
        self.imdb_id: str = utils.parse_value(str, data.get('imdb'))
        self.episode_count: int = utils.parse_value(int, data.get('total_ep_count'))
        self.season_index: int = utils.parse_value(int, data.get('season_index'))


@json_object
class DoubanSearchResult:
    id: int
    cn_name: str
    rating: float
    url: str
    app_url: str
    sub_id: int
    status: SubStatus

    def __init__(self, data: Dict):
        utils.copy_value(data, self)
        self.poster_url: str = utils.parse_value(str, data.get('poster_path'))


@json_object
class DoubanRankingType(Enum):
    movie_top250 = '豆瓣电影Top250'
    movie_real_time_hotest = '实时热门电影'
    movie_weekly_best = '一周口碑电影榜'
    ECPE465QY = '近期热门电影榜'
    EC7Q5H2QI = '近期高分电影榜'

    tv_chinese_best_weekly = '华语口碑剧集榜'
    tv_global_best_weekly = '全球口碑剧集榜'
    show_chinese_best_weekly = '国内口碑综艺榜'
    show_global_best_weekly = '国外口碑综艺榜'

    ECFA5DI7Q = '近期热门美剧'
    EC74443FY = '近期热门大陆剧'
    ECNA46YBA = '近期热门日剧'
    ECBE5CBEI = '近期热门韩剧'

    ECAYN54KI = '近期热门喜剧'
    ECBUOLQGY = '近期热门动作'
    ECSAOJFTA = '近期热门爱情'
    ECZYOJPLI = '近期热门科幻'
    EC3UOBDQY = '近期热门动画'
    ECPQOJP5Q = '近期热门悬疑'


@json_object
class DoubanRankingItem:
    rank: int
    id: int
    poster_path: str
    background_url: str
    cn_name: str
    release_year: str
    media_type: MediaType
    rating: float
    url: str
    app_url: str
    desc: str
    comment: str

    def __init__(self, data: Dict):
        utils.copy_value(data, self)
        self.media_type = MediaType.get(data.get('type'))


@json_object
class ApiSearchItem:
    media_type: MediaType
    douban_id: int
    rating: float
    title: str
    app_url: str
    small_poster_url: str
    year: int

    def __init__(self, data: Dict):
        if data.get('target_type'):
            self.media_type = MediaType.Movie if data.get('target_type') == 'movie' else MediaType.TV
        self.douban_id = utils.parse_value(int, data.get('target_id'))
        if data.get('target'):
            t = data.get('target')
            self.title = utils.parse_value(str, t.get('title'))
            self.app_url = utils.parse_value(str, t.get('uri'))
            self.small_poster_url = utils.parse_value(str, t.get('cover_url'))
            self.year = utils.parse_value(int, t.get('year'))
            self.rating = utils.parse_value(float, t.get('rating').get('value') if t.get('rating') else None)


@json_object
class DoubanDailyMedia:
    show_date: str
    media_type: str
    media_id: int
    douban_id: int
    tmdb_id: int
    title: str
    comment: str
    release_year: int
    rating: float
    poster_url: str
    background_url: str
    url: str
    app_url: str

    def __init__(self, data: Dict):
        utils.copy_value(data, self, True)


class DoubanApi:
    def __init__(self, session: Session):
        self._session: Session = session

    def get(self, douban_id: int) -> Optional[DoubanMedia]:
        """
        根据豆瓣编号获取豆瓣详情信息
        """
        meta = self._session.get('douban.get', {
            'douban_id': douban_id
        })
        if not meta:
            return
        return DoubanMedia(meta)

    def search(self, keyword: str) -> List[DoubanSearchResult]:
        """
        搜索豆瓣移动端网页解析结果，此搜索结果中没有年份
        """
        result = self._session.get('movie.search_douban', {
            'keyword': keyword
        })
        if not result:
            return []
        return [DoubanSearchResult(x) for x in result]

    def list_ranking(self, ranking_type: DoubanRankingType, proxy_pic: bool = False) -> Optional[
        List[DoubanRankingItem]]:
        """
        获取豆瓣榜单数据
        """
        res = self._session.get('douban.list_ranking', {
            'ranking_type': ranking_type.value,
            'proxy_pic': proxy_pic
        })
        if not res:
            return []
        return [DoubanRankingItem(x) for x in res.get('result')]

    def use_api_search(self, keyword: str, count: Optional[int] = None) -> List[ApiSearchItem]:
        """
        使用豆瓣移动端API进行搜索，此接口会含有年份信息
        """
        res = self._session.get('douban.use_api_search', {
            'keyword': keyword,
            'count': count
        })
        if not res or not res.get('items'):
            return []
        list_ = res.get('items')
        result = []
        for item in list_:
            result.append(ApiSearchItem(item))
        return result

    def daily_media(self) -> Optional[DoubanDailyMedia]:
        """
        获取豆瓣每日推荐
        """
        meta = self._session.get('common.daily_media')
        if not meta:
            return
        return DoubanDailyMedia(meta)



from typing import Optional, Dict, List

from moviebotapi import Session
from moviebotapi.core import utils
from moviebotapi.core.utils import json_object


@json_object
class MenuSubItem:
    href: str
    icon: str
    title: str

    def __init__(self, data: Dict = None):
        if data:
            utils.copy_value(data, self)


@json_object
class MenuItem:
    href: str
    """
    icon的值就是MUI icon的名称
    https://mui.com/material-ui/material-icons/
    """
    icon: str
    title: str
    children: List[MenuSubItem]

    def __init__(self, data: Dict = None):
        if data:
            utils.copy_value(data, self)


@json_object
class MenuGroup:
    title: str
    pages: List[MenuItem]

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


class CommonApi:
    def __init__(self, session: Session):
        self._session: Session = session

    def restart_app(self, secs: Optional[int] = 3):
        self._session.get('common.restart_app', {
            'secs': secs
        })

    def get_cache(self, namespace: str, key: str) -> Dict:
        return self._session.get('common.get_cache', {
            'namespace': namespace,
            'key': key
        })

    def set_cache(self, namespace: str, key: str, data: Dict):
        self._session.post('common.set_cache', {
            'namespace': namespace,
            'key': key,
            'data': data
        })

    def get_image_text(self, b64_img: str):
        return self._session.post('common.get_image_text', {
            'b64_image': b64_img
        })

    def get_cache_image_filepath(self, img_url: str) -> str:
        return self._session.get('common.get_cache_image_filepath', {
            'url': img_url
        })

    def list_menus(self) -> List[MenuGroup]:
        items = self._session.get('common.get_menus')
        if not items:
            return []
        return [MenuGroup(x) for x in items]

    def save_menus(self, menus: List[MenuGroup]):
        if not menus:
            return
        self._session.post('common.save_menus', {
            'menus': [x.to_json() for x in menus]
        })



from abc import ABCMeta, abstractmethod
from typing import Union, Dict, Sequence, Tuple, Optional, Any

import httpx
from httpx import Timeout

from moviebotapi.core.exceptions import ApiErrorException, NetworkErrorException, IllegalAuthorization

UA = 'moviebotapi/0.0.55'
URLTypes = Union["URL", str]
HeaderTypes = Union[
    "Headers",
    Dict[str, str],
    Dict[bytes, bytes],
    Sequence[Tuple[str, str]],
    Sequence[Tuple[bytes, bytes]],
]


class Session(metaclass=ABCMeta):
    @staticmethod
    def un_code(res):
        if res:
            if res.get('code') == 0:
                return res.get('data')
            if res.get('code') == 1:
                raise ApiErrorException(res.get('message'))
        else:
            return

    @abstractmethod
    def post(self, api_code: str, json: Optional[Any] = None, headers: Optional[HeaderTypes] = None):
        pass

    @abstractmethod
    def get(self, api_code: str, params: Optional[Dict[str, Any]] = None, headers: Optional[HeaderTypes] = None):
        pass


class AccessKeySession(Session):
    def __init__(self, server_url: URLTypes, access_key: str):
        self.server_url: URLTypes = server_url
        self.access_key: str = access_key
        self.timeout = Timeout(30)

    def _get_headers(self, headers: Optional[HeaderTypes] = None):
        if not headers:
            headers = {}
        headers.update({'AccessKey': self.access_key})
        headers.update({'User-Agent': UA})
        return headers

    @staticmethod
    def _get_api_uri(api_code: str):
        if not api_code:
            return
        return '/api/' + api_code.replace('.', '/')

    def get(self, api_code: str, params: Optional[Dict[str, Any]] = None, headers: Optional[HeaderTypes] = None):
        r = httpx.get(f'{self.server_url}{self._get_api_uri(api_code)}', params=params,
                      headers=self._get_headers(headers),
                      timeout=self.timeout)
        if not r:
            raise NetworkErrorException()
        if r.status_code == 401:
            raise IllegalAuthorization()
        r.raise_for_status()
        return self.un_code(r.json())

    def post(self, api_code: str, json: Optional[Any] = None, headers: Optional[HeaderTypes] = None):
        r = httpx.post(f'{self.server_url}{self._get_api_uri(api_code)}', json=json, headers=self._get_headers(headers),
                       timeout=self.timeout)
        if not r:
            raise NetworkErrorException()
        r.raise_for_status()
        return self.un_code(r.json())



from typing import Dict, List, Optional

from moviebotapi import Session


class AuthApi:
    def __init__(self, session: Session):
        self._session: Session = session

    def get_default_ak(self, ) -> Dict:
        return self._session.get('auth.get_default_ak')

    def add_permission(self, role_code: List[int], uri: str):
        """
        为角色授权可访问的URI
        :param role_code: 角色码 1为管理员 2为普通用户，目前固定不变
        :param uri: 权限点URI
        :return:
        """
        self._session.post('auth.add_permission', {
            'role_code': role_code,
            'uri': uri
        })

    def get_cloud_access_token(self) -> Optional[str]:
        return self._session.get('auth.get_cloud_access_token')



from typing import Dict, List

from moviebotapi import Session
from moviebotapi.core import utils


class PluginMeta:
    id: int
    plugin_name: str
    title: str
    author: str
    config_field: list
    dependencies: dict
    description: str
    github_url: str
    help_doc_url: str
    local_version: str
    logo_url: str
    version: str
    plugin_folder: str

    def __init__(self, data: Dict):
        utils.copy_value(data, self, True)


class PluginApi:
    def __init__(self, session: Session):
        self._session: Session = session

    def get_installed_list(self) -> List[PluginMeta]:
        """
        获取本地安装的插件列表
        """
        r = self._session.get('plugins.get_installed_list')
        if not r:
            return []
        res = []
        for item in r:
            res.append(PluginMeta(item))
        return res



from moviebotapi.core.basemodel import BaseModel


class ClientTorrent(BaseModel):
    hash: str
    name: str
    save_path: str
    content_path: str
    size: int
    size_str: str
    dl_speed: int
    dl_speed_str: str
    up_speed: int
    up_speed_str: str
    progress: float
    uploaded: int
    uploaded_str: str
    downloaded: int
    downloaded_str: str
    seeding_time: int
    ratio: float
    tracker: str



def ignore_attr_not_exists(cls):
    """
    忽略属性不存在的异常，正常返回空结果
    """
    orig_getattribute = cls.__getattribute__

    def getattribute(self, attr):
        if not hasattr(cls, attr):
            return
        return orig_getattribute(self, attr)

    cls.__getattribute__ = getattribute
    return cls



from typing import Dict

from moviebotapi import Session


class EventApi:
    def __init__(self, session: Session):
        self._session: Session = session

    def publish_event(self, event_name: str, event_data: Dict):
        """
        发布事件，订阅者可以订阅
        事件名称建议采用每个单词首字母大写的方式来命名
        """
        self._session.post('event.publish_event', {
            'event_name': event_name,
            'event_data': event_data
        })
